declare enum TypesFn {
    fn = 0,
    arrowFn = 1,
    propFn = 2
}
declare function getHeaderFn(code: string): {
    name: string;
    args: string;
    content: string | undefined;
    isAsync: boolean;
    isStar: boolean;
    typeFn: TypesFn;
};

interface RealItem<T> {
    "@hidden": boolean;
    "@value": T;
}
declare namespace Data {
    interface Link {
        "@t": "link";
        "@type": "object" | "function";
        "@link": string;
        "@name": string | null;
    }
    interface String {
        "@t": "string";
        "@first": boolean;
        "@value": string;
    }
    interface Number {
        "@t": "number";
        "@value": string;
    }
    interface BigInt {
        "@t": "bint";
        "@value": string;
    }
    interface Symbol {
        "@t": "symbol";
        "@value": string;
    }
    interface Nill {
        "@t": "nill";
        "@value": "null" | "undefined";
    }
    interface Function {
        "@t": "function";
        "@first": boolean;
        "@header": ReturnType<typeof getHeaderFn>;
        "@code": string;
        "@real": Link | null;
    }
    interface Collection extends Omit<Record, "@t" | "@des" | "@first" | "@real"> {
        "@t": "collection";
        "@name": "map" | "weakmap" | "set" | "weakset";
        "@size": number | null;
        "@entries": unknown;
        "@real": Link;
    }
    interface RegExp extends Omit<Record, "@t" | "@de" | "@real" | "@des"> {
        "@t": "regexp";
        "@flags": string;
        "@source": string;
        "@real": Link | null;
    }
    interface GetSetter {
        "@t": "gs";
        "@value": Link;
        "@at": Partial<{
            [name in "get" | "set"]: Function;
        }>;
    }
    interface objReal {
        [name: string]: RealItem<GetSetter | String | Number | BigInt | Symbol | Function | Collection | RegExp | Record | Nill | Error | Array | Element | Promise | Date | TypedArray | Buffer | DataView>;
    }
    interface Record {
        "@t": "object";
        "@name": string | null;
        "@first": boolean;
        "@real": objReal | Link;
        "@des": {
            "@value": DataPreview.objReal;
            "@lastKey": string;
        } | null;
    }
    interface Error {
        "@t": "error";
        "@first": boolean;
        "@stack": string;
        "@real": Link | null;
    }
    interface Array extends Pick<Record, "@des"> {
        "@t": "array";
        "@size": number;
        "@name": string | null;
        "@first": boolean;
        "@real": Link;
    }
    type ArrayReal = objReal & {
        length: RealItem<Number>;
    };
    interface Element {
        "@t": "element";
        "@name": string;
        "@first": boolean;
        "@attrs"?: [string, string][];
        "@real": Link | null;
        "@childs"?: string | null | Link;
    }
    interface Promise {
        "@t": "promise";
        "@first": boolean;
        "@state": "pending" | "fulfilled" | "rejected";
        "@real": Link;
        "@des": Exclude<Record["@des"], null>;
    }
    interface Date extends Pick<Record, "@des"> {
        "@t": "date";
        "@first": boolean;
        "@value": string;
        "@real": Link | null;
    }
    interface TypedArray extends Omit<Array, "@t"> {
        "@t": "typedarray";
        "@real": Link;
    }
    type TypedArrayReal = ArrayReal & {
        buffer: RealItem<Buffer>;
        byteLength: RealItem<Number>;
        byteOffset: RealItem<Number>;
        [Symbol.toStringTag]: RealItem<String>;
    };
    interface Buffer extends Omit<Array, "@t" | "@des" | "@real"> {
        "@t": "buffer";
        "@real": Link;
    }
    type BufferReal = objReal & {
        byteLength: RealItem<Number>;
        "[[Int8Array]]": RealItem<TypedArray>;
        "[[Uint8Array]]": RealItem<TypedArray>;
        "[[Int16Array]]": RealItem<TypedArray>;
        "[[Int32Array]]": RealItem<TypedArray>;
        "[[ArrayBufferByteLength]]": RealItem<Number>;
    };
    interface DataView extends Omit<Array, "@t" | "@des" | "@name"> {
        "@t": "dataview";
        "@name": "DataView";
        "@real": Link;
    }
    type DataViewReal = ArrayReal & {
        buffer: RealItem<Buffer>;
        byteLength: RealItem<Number>;
        byteOffset: RealItem<Number>;
    };
}
declare function readLinkObject(link: Data.Link): Data.Record | Data.Error | Data.RegExp | Data.Collection | Data.Array | Data.Function | Data.Element | Data.Promise | Data.Date | Data.TypedArray | Data.Buffer | Data.DataView | Data.String | Data.Number | Data.BigInt | Data.Symbol | Data.Nill;
declare function callFnLink(link: Data.Link): ReturnType<typeof _Encode> | Data.Error;
declare function _getListLink(link: Data.Link): (ReturnType<typeof _Encode> | Data.Error)[];
declare function clearLinkStore(): void;
declare function _Encode(data: unknown, first: boolean, // false,
linkReal: boolean): Data.String | Data.Number | Data.BigInt | Data.Symbol | Data.Function | Data.Collection | Data.RegExp | Data.Nill | Data.Record | Data.Error | Data.Array | Data.Element | Data.Promise | Data.Date | Data.TypedArray | Data.Buffer | Data.DataView;
declare function Encode(data: unknown, deepLink?: false | number, first?: boolean, linkReal?: boolean): {
    "@location": string | null;
    "@t": "object";
    "@name": string | null;
    "@first": boolean;
    "@real": Data.objReal | Data.Link;
    "@des": {
        "@value": DataPreview.objReal;
        "@lastKey": string;
    } | null;
} | {
    "@location": string | null;
    "@t": "error";
    "@first": boolean;
    "@stack": string;
    "@real": Data.Link | null;
} | {
    "@location": string | null;
    "@t": "regexp";
    "@flags": string;
    "@source": string;
    "@real": Data.Link | null;
    "@name": string | null;
    "@first": boolean;
} | {
    "@location": string | null;
    "@t": "collection";
    "@name": "map" | "weakmap" | "set" | "weakset";
    "@size": number | null;
    "@entries": unknown;
    "@real": Data.Link;
} | {
    "@location": string | null;
    "@t": "array";
    "@size": number;
    "@name": string | null;
    "@first": boolean;
    "@real": Data.Link;
    "@des": {
        "@value": DataPreview.objReal;
        "@lastKey": string;
    } | null;
} | {
    "@location": string | null;
    "@t": "function";
    "@first": boolean;
    "@header": {
        name: string;
        args: string;
        content: string | undefined;
        isAsync: boolean;
        isStar: boolean;
        typeFn: TypesFn;
    };
    "@code": string;
    "@real": Data.Link | null;
} | {
    "@location": string | null;
    "@t": "element";
    "@name": string;
    "@first": boolean;
    "@attrs"?: [string, string][] | undefined;
    "@real": Data.Link | null;
    "@childs"?: string | Data.Link | null | undefined;
} | {
    "@location": string | null;
    "@t": "promise";
    "@first": boolean;
    "@state": "pending" | "fulfilled" | "rejected";
    "@real": Data.Link;
    "@des": {
        "@value": DataPreview.objReal;
        "@lastKey": string;
    };
} | {
    "@location": string | null;
    "@t": "date";
    "@first": boolean;
    "@value": string;
    "@real": Data.Link | null;
    "@des": {
        "@value": DataPreview.objReal;
        "@lastKey": string;
    } | null;
} | {
    "@location": string | null;
    "@t": "typedarray";
    "@real": Data.Link;
    "@name": string | null;
    "@size": number;
    "@des": {
        "@value": DataPreview.objReal;
        "@lastKey": string;
    } | null;
    "@first": boolean;
} | {
    "@location": string | null;
    "@t": "buffer";
    "@real": Data.Link;
    "@name": string | null;
    "@size": number;
    "@first": boolean;
} | {
    "@location": string | null;
    "@t": "dataview";
    "@name": "DataView";
    "@real": Data.Link;
    "@size": number;
    "@first": boolean;
} | {
    "@location": string | null;
    "@t": "string";
    "@first": boolean;
    "@value": string;
} | {
    "@location": string | null;
    "@t": "number";
    "@value": string;
} | {
    "@location": string | null;
    "@t": "bint";
    "@value": string;
} | {
    "@location": string | null;
    "@t": "symbol";
    "@value": string;
} | {
    "@location": string | null;
    "@t": "nill";
    "@value": "undefined" | "null";
};
declare namespace DataPreview {
    type Record = Pick<Data.Record, "@t" | "@name">;
    type Error = Pick<Data.Error, "@t" | "@stack">;
    type RegExp = Pick<Data.RegExp, "@t" | "@name">;
    type Collection = Pick<Data.Collection, "@t" | "@name" | "@size">;
    type Array = Pick<Data.Array, "@t" | "@size" | "@name">;
    type Function = Pick<Data.Function, "@t">;
    type Element = Pick<Data.Element, "@t" | "@name">;
    type Promise = Pick<Data.Promise, "@t">;
    type Date = Pick<Data.Date, "@t" | "@value">;
    type TypedArray = Pick<Data.TypedArray, "@t" | "@size" | "@name">;
    type Buffer = Pick<Data.Buffer, "@t" | "@size" | "@name">;
    type DataView = Pick<Data.DataView, "@t" | "@size" | "@name">;
    interface objReal {
        [name: string]: RealItem<Record | Error | RegExp | Collection | Array | Function | Element | Promise | Date | TypedArray | Buffer | DataView | Data.String | Data.Number | Data.BigInt | Data.Symbol | Data.Nill>;
    }
}
declare function createPreviewValue(value: unknown): DataPreview.objReal[""]["@value"];

export { Data, DataPreview, Encode, _Encode, _getListLink, callFnLink, clearLinkStore, createPreviewValue, readLinkObject };
